package entDownloader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.UnknownHostException;
import java.net.HttpURLConnection;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.GregorianCalendar;
import java.util.List;


public class ENTDownloader {
	
	/** L'instance statique */
	private static ENTDownloader instance;
	
	/** Flag indiquant si l'utilisateur est connect ou non */
	private boolean isLogin = false;
	
	/** Liste des dossiers et des fichiers du dossier courant */
	private List<FS_Element> directoryContent = null;
	/** Chemin vers le dossier courant */
	private /*Stack<String>*/ ENTPath path = null;
	
	/** Paramtre de l'URL de la page de stockage **/
	private String uP_root; 
	/** Paramtre de l'URL de la page de stockage **/
	private String tag;
	/** Identifiant de connexion */
	private String sessionid;
	
	/** Instance d'un navigateur utilis pour la communication avec le serveur */
	private Browser browser;
	
	/** 
	 * Rcupre l'instance unique de la class ENTDownloader.<br>
	 * Remarque : le constructeur est rendu inaccessible
	 */
	public static ENTDownloader getInstance() {
	    if (null == instance) { // Premier appel
	        instance = new ENTDownloader();
	    }
	    return instance;
	}
	
	private ENTDownloader() {
		browser = new Browser();
	}
	
	/**
	 * tablit la connexion au serveur de l'ENT.
	 * @param username Le nom d'utilisatateur pour la connexion
	 * @param password Mot de passe de connexion
	 * @return True en cas de russite, ou false si l'authentification a chou
	 */
	public boolean login(String username, String password) throws java.net.UnknownHostException {
		if (isLogin == true) return true;
		
		browser.setUrl(Config.loginURL);
		
		String loginPage = browser.perform();
		List<String> ticket = new ArrayList<String>();
		Misc.preg_match("input type=\"hidden\" name=\"lt\" value=\"([0-9a-zA-Z\\-]+)\" />", loginPage, ticket);
		
		browser.setMethod(Browser.Method.POST);
		browser.setParam("username", username);
		browser.setParam("password",password);
		browser.setParam("lt", ticket.get(1).toString());
		browser.setFollowRedirects(false);
		loginPage = browser.perform();
		browser.setMethod(Browser.Method.GET);
		
		if (Misc.preg_match("<div id=\"erreur\">", loginPage, ticket)) {
			return false;
		}
		Misc.preg_match("<a href=\"(.*)\">ici", loginPage, ticket);
		
		browser.setUrl(ticket.get(1).toString());
		browser.clearParam();
		browser.perform();
		sessionid = browser.getCookieField("JSESSIONID");
		
		isLogin = true;
		
		try {
			setStockageUrlParams();
		} catch (ParseException e) {
			e.printStackTrace();
		}
		directoryContent = null;
		path = null;
		return true;
	}
	
	private void setStockageUrlParams() throws ParseException{
		if (isLogin == false)
			throw new ENTUnauthenticatedUserException("Non-authenticated user.", ENTUnauthenticatedUserException.UNAUTHENTICATED);
		
		browser.setFollowRedirects(true);
		browser.setUrl(Config.rootURL);
		browser.clearParam();
		String rootPage = null;
		try {
			rootPage = browser.perform();
		} catch (UnknownHostException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		setStockageUrlParams(rootPage);
	}
	
	private void setStockageUrlParams(String pageContent) throws ParseException{
		List<String> matches = new ArrayList<String>(5);
		if (!Misc.preg_match("<a href=\"[\\w\\d:#@%/;$()~_?\\+-=\\\\.&]*?/tag\\.([0-9A-Fa-f]{13,17})\\.[\\w\\d:#@%/;$()~_?\\+-=\\\\.&]*?uP_root=([\\w\\|]+?)&[\\w\\d:#@%/;$()~_?\\+-=\\\\.&]*?\" title=\"Espace de stockage WebDAV\" id=\"chanLink\"><span>Mes Documents</span>", pageContent, matches))
			throw new ParseException("Unable to find the URL parameters of the storage's service in this page.", -1);
		tag = matches.get(1);
		uP_root = matches.get(2);
	}
	
	/**
	 * Change le rpertoire courant
	 * @param name Nom du dossier ou directive de parcours. Le dossier . est le dossier courant : appeler cette methode avec ce paramtre ne change donc pas le dossier courant
	 * 				mais permet de rafrachir son contenu. Le dossier .. est le dossier parent, il permet donc de remonter dans l'arborescence. Enfin, le dossier / est la racine
	 *				du service de stockage de l'utilisateur.
	 * @throws ENTDirectoryNotFoundException Si le rpertoire demand n'existe pas dans le dossier courant
	 * @throws ENTInvalidFS_ElementTypeException Si le nom d'un fichier a t pass en paramtre.
	 * @throws ParseException En cas d'erreur d'analyse du contenu du dossier cible.
	 * @throws ENTUnauthenticatedUserException Si l'utilisateur n'est pas authentifier lors de l'appel de la mthode
	 */
	public void changeDirectory(String path) throws ENTUnauthenticatedUserException, ENTDirectoryNotFoundException, ENTInvalidFS_ElementTypeException, ParseException {
		if (path == null) throw new NullPointerException();
		if (path.isEmpty()){
			submitDirectory("");
			return;
		}
		
		String splitPath[] = path.split("/");
		if(splitPath.length>1){ //Si le chemin de destination est compos, on cherche  le simplifier
			ENTPath destination = new ENTPath(this.path); destination.goTo(path);
			String absDest = destination.toString(), relDest = this.path.getRelative(destination);
			int absc = destination.getNbRequests(), relc = ENTPath.getNbRequests(relDest);
			if(relc<absc)
				path = relDest;
			else
				path = absDest;
			splitPath = path.split("/");
		}
		
		boolean isAbsolute = ENTPath.isAbsolute(path);

		if (isAbsolute)
			submitDirectory("/");
		
		for (int i = isAbsolute?1:0; i < splitPath.length; ++i) {
			if(!splitPath[i].isEmpty() && (i == 0 || !splitPath[i].equals("."))) {
				/* Corrige les slash multiple dans le chemin et ignore les .  l'intrieur :
				 * 		./toto/././////./titi est trait comme ./toto/titi
				 */
					submitDirectory(splitPath[i]);
			}
		}
	}
	
	/**
	 * Descend ou remonte d'un pas dans l'aborescence  partir du dossier courant.
	 * @param name Nom du dossier ou directive de parcours. Le dossier . est le dossier courant, appeler cette methode avec ce paramtre ne change donc pas le dossier courant
	 * 				mais permet de rafrachir son contenu. Le dossier .. est le dossier parent, il permet donc de remonter dans l'arborescence. Enfin, le dossier / est la racine
	 *				du service de stockage de l'utilisateur.
	 * @throws ENTDirectoryNotFoundException Si le rpertoire demand n'existe pas dans le dossier courant
	 * @throws ENTInvalidFS_ElementTypeException Si le nom d'un fichier a t pass en paramtre.
	 * @throws ParseException {@link ENTDownloader#parsePage(String) Voir la mthode parsePage}
	 * @throws ENTUnauthenticatedUserException Si l'utilisateur n'est pas authentifier lors de l'appel de la mthode
	 */
	private void submitDirectory(String name) throws ENTDirectoryNotFoundException, ENTInvalidFS_ElementTypeException, ParseException, ENTUnauthenticatedUserException {
		if (isLogin == false)
			throw new ENTUnauthenticatedUserException("Non-authenticated user.",ENTUnauthenticatedUserException.UNAUTHENTICATED);
		
		browser.clearParam();
		if (name.equals("/") || name.equals("~") || name.equals("")) {
			if(path == null) {
				path = new ENTPath();
				browser.setUrl(urlBuilder(Config.stockageURL));
			}
			else {
				path.clear();
				browser.setUrl(urlBuilder("http://ent.u-clermont1.fr/tag.{tag}.render.userLayoutRootNode.target.{uP_root}.uP?link=0#{uP_root}"));
			}
			name = "/";
			browser.setMethod(Browser.Method.GET);
		}
		else if (name.equals(".")) {
			browser.setMethod(Browser.Method.GET);
			browser.setUrl(urlBuilder(Config.refreshDirURL));
		}
		else if (name.equals("..")) {
			if (path.isRoot()) return; //Dj  la racine
			browser.setMethod(Browser.Method.GET);
			browser.setUrl(urlBuilder(Config.directoryBackURL));
		}
		else {
			int pos = indexOf(name);
			if (pos == -1)
				throw new ENTDirectoryNotFoundException("Directory not found");
			else if (!directoryContent.get(pos).isDirectory())
				throw new ENTInvalidFS_ElementTypeException(name + " isn't a directory");
			
			browser.setUrl(urlBuilder(Config.goIntoDirectoryURL));
			browser.setMethod(Browser.Method.POST);
			browser.setParam("targetDirectory", HTMLEntities.htmlentities(name));
		}
		browser.setFollowRedirects(false);
		browser.setCookieField("JSESSIONID", sessionid);
		String pageContent = null;
		try {
			pageContent = browser.perform();
		} catch (UnknownHostException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		if(browser.getResponseCode() == HttpURLConnection.HTTP_MOVED_TEMP && browser.getHeaderField("Location").equals(Config.loginRequestURL)){
			isLogin = false;
			throw new ENTUnauthenticatedUserException("Session expired, please login again.", ENTUnauthenticatedUserException.SESSION_EXPIRED);
		}
		
		setStockageUrlParams(pageContent);

		if (pageContent.isEmpty() || Misc.preg_match("<font class=\"uportal-channel-strong\">La ressource sp&eacute;cifi&eacute;e n'existe pas.<br /></font>", pageContent))
			throw new ENTDirectoryNotFoundException(name + ": Directory not found");
		parsePage(pageContent);
		
		path.goTo(name);
	}
	
	/**
	 * Tlcharge le fichier <i>name</i>. 
	 * Le fichier sera enregistr sous le dossier et le nom spcifi dans le paramtre <i>destination</i>, ou sous le mme nom que celui sous lequel il est stock sur l'ENT si le nouveau nom n'est pas indiqu dans <i>destination</i>.<br><br>
	 * Exemples pour un fichier "tp13.pdf":<br><ul>
	 * <li>Si le chemin de destination est "/home/sasa/tps/tpNumero13.pdf", le fichier sera stock sous "/home/sasa/tps/tpNumero13.pdf";</li>
	 * <li>Si le chemin de destination est "/home/sasa/bonjour/", le fichier sera stock sous "/home/sasa/bonjour/tp13.pdf";</li>
	 * <li>Si le chemin de destination est "tp13.pdf", le fichier sera stock sous System.getProperty("user.dir") + "tp13.pdf";</li>
	 * <li>Si le chemin de destination est vide ou null, le fichier sera stock sous System.getProperty("user.dir") + nom utilis sous l'ENT;</li>
	 * <li>Si le chemin de destination est "~/bonjour.pdf", le fichier sera stock sous System.getProperty("user.home") + "bonjour.pdf;</li>
	 * </ul>
	 * @param name Nom du fichier  tlcharger
	 * @param destination Chemin de destination du fichier
	 * @throws IOException
	 * @see Misc#tildeToHome(String)
	 * @see Browser#downloadFile(String)
	 */
	public void getFile(String name, String destination) throws IOException {
		if (isLogin == false)
			throw new ENTUnauthenticatedUserException("Non-authenticated user.",ENTUnauthenticatedUserException.UNAUTHENTICATED);
		int pos = indexOf(name);
		if (pos == -1)
			throw new ENTFileNotFoundException("File not found");
		else if (!directoryContent.get(pos).isFile())
			throw new ENTInvalidFS_ElementTypeException(name + " isn't a file");
		
		if (destination == null || destination.isEmpty()) {
			destination = name;
		} else if (!destination.equals(name)) {
			destination = Misc.tildeToHome(destination);
			if(destination.substring(destination.length()-1).equals(System.getProperty("file.separator")))
				destination += name;
		}
		
		browser.clearParam();
		browser.setUrl(urlBuilder(Config.downloadFileURL));
		browser.setMethod(Browser.Method.POST);
		browser.setParam("downloadFile", HTMLEntities.htmlentities(name));
		browser.setFollowRedirects(false);
		browser.setCookieField("JSESSIONID", sessionid);
		browser.downloadFile(destination);
	}
	
	/**
	 * Tlcharge le fichier <i>name</i>. 
	 * Le fichier sera enregistr dans le dossier local courant (gnralement le dossier de l'application), sous le mme nom que celui sous lequel il est stock sur l'ENT.
	 * @param name Le nom du fichier  tlcharger.
	 * @throws IOException
	 */
	public void getFile(String name) throws IOException {
		getFile(name, name);
	}
	
	/**
	 * Tlcharge tous les fichiers contenus dans le dossier courant et ses sous dossiers. 
	 * Les fichiers et dossiers seront enregistrs sous le dossier <i>destination</i>, sous le mme nom que celui sous lequel il sont stocks sur l'ENT.
	 * @param destination Dossier de destination des fichiers et dossiers tlchargs. Si null ou vide, ils seront enregistrs dans le rpertoire courant.
	 * @throws ENTInvalidFS_ElementTypeException Lance lorsque le paramtre <i>destination</i> dsigne un fichier existant.
	 * @throws IOException
	 * @see ENTDownloader#getFile(String, String)
	 * @return Le nombre de fichiers tlchargs
	 * @deprecated Remplac par getAllFile(String destination, int maxdepth)
	 */
	@Deprecated public int getAllFiles(String destination) throws IOException {
		return getAllFiles(destination, -1);
	}
	
	/**
	 * Tlcharge tous les fichiers contenus dans le dossier courant et ses sous dossiers. 
	 * Les fichiers et dossiers seront enregistrs sous le dossier <i>destination</i>, sous le mme nom que celui sous lequel il sont stocks sur l'ENT.
	 * @param destination Dossier de destination des fichiers et dossiers tlchargs. Si null ou vide, ils seront enregistrs dans le rpertoire courant.
	 * @param maxdepth Profondeur maximale de tlchargement. 0 (zro) signifie que la mthode ne va tlcharger que les fichiers du dossier courant, sans descendre dans les sous-dossiers. Une valeur ngative signifie aucune limite.
	 * @throws ENTInvalidFS_ElementTypeException Lance lorsque le paramtre <i>destination</i> dsigne un fichier existant.
	 * @throws IOException
	 * @see ENTDownloader#getFile(String, String)
	 * @return Le nombre de fichiers tlchargs
	 */
	public int getAllFiles(String destination, int maxdepth) throws IOException {
		int i=0;
		if (directoryContent == null)
			throw new IllegalStateException("Directory content hasn't been initialized");
		if (destination == null)
			destination = "";
		else if (!destination.isEmpty()) {
			if(!destination.substring(destination.length()-1).equals(System.getProperty("file.separator")))
				destination += System.getProperty("file.separator");
			destination = Misc.tildeToHome(destination);
			File dest = new File(destination);
			if (dest.isFile())
				throw new ENTInvalidFS_ElementTypeException("Unable to create the required directory : a file with that name exists.");
		}
		List<FS_Element> directoryContentcp = new ArrayList<FS_Element>(directoryContent);
		for(FS_Element e : directoryContentcp)
			if (e.isFile()){
				getFile(e.getName(), destination);
				++i;
			}
			else if (maxdepth != 0) {
				try {
					submitDirectory(e.getName());
				} catch (ParseException e1) {
					try {
						submitDirectory(e.getName());
					} catch (ParseException e2) {
						e2.printStackTrace();
					}
				}
				i+= getAllFiles(destination + e.getName(), maxdepth-1);
				try {
					submitDirectory("..");
				} catch (ParseException e1) {
					try {
						submitDirectory("..");
					} catch (ParseException e2) {
						e2.printStackTrace();
					}
				}
			}
		return i;
	}
	
	/**
	 * Analyse le contenu de la page pass en paramtre afin de dterminer les dossiers et fichiers contenus dans le dossier courant.
	 * @param pageContent Le contenu de la page  analyser
	 * @throws ParseException Si l'analyse choue
	 */
	private void parsePage(String pageContent) throws ParseException {
		List<List<String>> matches = new ArrayList<List<String>>();
		if (directoryContent == null)
			directoryContent = new ArrayList<FS_Element>(50);
		else
			directoryContent.clear();
		
		//Misc.preg_match_all("&nbsp;<a href=\"javascript:submit(File|Directory)\\('(.+?)'\\);\"\\s+class.*?</a></td><td class=\"uportal-crumbtrail\" align=\"right\">\\s+?&nbsp;([0-9][0-9]*\\.[0-9][0-9]? [MKGo]{1,2})?\\s*?</td><td class=\"uportal-crumbtrail\" align=\"right\">\\s+?&nbsp;([0-9]{2})-([0-9]{2})-([0-9]{4})&nbsp;([0-9]{2}):([0-9]{2})", pageContent, matches, Misc.PREG_ORDER.SET_ORDER); //Erreur d'chappement de caractres spciaux dans le paramtre javascript
		Misc.preg_match_all("&nbsp;<a href=\"javascript:submit(File|Directory)\\('.+?'\\);\"\\s+class.*?nnel\">(.*?)</a></td><td class=\"uportal-crumbtrail\" align=\"right\">\\s+?&nbsp;([0-9][0-9]*\\.[0-9][0-9]? [MKGo]{1,2})?\\s*?</td><td class=\"uportal-crumbtrail\" align=\"right\">\\s+?&nbsp;([0-9]{2})-([0-9]{2})-([0-9]{4})&nbsp;([0-9]{2}):([0-9]{2})", pageContent, matches, Misc.PREG_ORDER.SET_ORDER);
		for(List<String> fileInfos : matches){
			FS_Element file = null; // Fichier ou rpertoire
			if (fileInfos.get(1).equals("Directory"))
				file = new FS_Directory(HTMLEntities.unhtmlentities(fileInfos.get(2)), new GregorianCalendar(Integer.parseInt(fileInfos.get(6)), Integer.parseInt(fileInfos.get(5)), Integer.parseInt(fileInfos.get(4)), Integer.parseInt(fileInfos.get(7)), Integer.parseInt(fileInfos.get(8))));
			else if (fileInfos.get(1).equals("File"))
				file = new FS_File(HTMLEntities.unhtmlentities(fileInfos.get(2)), new GregorianCalendar(Integer.parseInt(fileInfos.get(6)), Integer.parseInt(fileInfos.get(5)), Integer.parseInt(fileInfos.get(4)), Integer.parseInt(fileInfos.get(7)), Integer.parseInt(fileInfos.get(8))), fileInfos.get(3));
			if (file == null)
				throw new ParseException("Error while parsing page content : unable to determine if \""+fileInfos.get(2)+"\" is a file or a directory.", -1);
			directoryContent.add(file);
		}
	}

	/**
	 * Construit l'url demand en remplaant les champs {...} par les valeurs correspondantes
	 * @param url Url  construire
	 * @return L'url pass en paramtre, aprs avoir remplacer les champs {...}.
	 */
	private String urlBuilder(String url){
		return url.replaceAll("\\{tag\\}", tag).replaceAll("\\{uP_root\\}", uP_root);
	}
	
	/**
	 * @return Le contenu du rpertoire courant
	 */
	public List<FS_Element> getDirectoryContent() {
		if (directoryContent == null)
			throw new IllegalStateException("Directory content hasn't been initialized");
		return directoryContent;
	}
	
	/**
	 * @return Le nom du rpertoire courant
	 */
	public String getDirectoryName() {
		/*if (directoryContent == null)
			throw new IllegalStateException("Current directory is undefined");
		return path.peek();*/
		return path.getDirectoryName();
	}
	
	/**
	 * @return Le nombre de dossiers dans le dossier courant.
	 */
	public int getNbDossiers() {
		int i = 0;
		for (FS_Element e : directoryContent) {
			if (e.isDirectory())
				++i;
		}
		return i;
	}

	/**
	 * @return Le nombre de fichiers dans le dossier courant.
	 */
	public int getNbFiles() {
		int i = 0;
		for (FS_Element e : directoryContent) {
			if (e.isFile())
				++i;
		}
		return i;
	}

	/**
	 * @return La taille totale des fichiers dans le dossier courant en octets.
	 */
	public long getFilesSize() {
		long s = 0;
		for (FS_Element e : directoryContent) {
			if (e.isFile())
				s+= ((FS_File) e).getSize();
		}
		return s;
	}
	
	/**
	 * @return Le chemin absolu permettant d'atteindre le rpertoire courant
	 */
	public String getDirectoryPath() {
		/*if (directoryContent == null)
			throw new IllegalStateException("Current directory is undefined");
		String path = this.path.getLast(); //Racine
		for (int i = this.path.size() -2; i>=0; --i) {
			//if(i != this.path.size() -2)
			path+="/";
			path+= this.path.get(i);
		}
		return path;
		*/
		return path.toString();
	}
	
	/**
	 * Returns the index of the first occurrence of the specified element in the {@link  entDownloader.ENTDownloader#directoryContent directoryContent} list, or -1 if this list does not contain the element.
	 * More formally, returns the lowest index i such that (o==null ? get(i)==null : get(i).equals(o)), or -1 if there is no such index. 
	 * @param o element to search for
	 * @return the index of the first occurrence of the specified element in the {@link  entDownloader.ENTDownloader#directoryContent directoryContent} list, or -1 if this list does not contain the element.
	 * @throws IllegalStateException if the directory content hasn't been initialized.
	 */
	public int indexOf(Object o) throws IllegalStateException{
		if (directoryContent == null)
			throw new IllegalStateException("Directory content hasn't been initialized");
		int i = 0;
		for (Object e : directoryContent) {
			if (e.equals(o))
				return i;
			++i;
		}
		return -1;
	}
}

/**
 * Signals that an attempt to open the directory denoted by a specified name has failed
 */
class ENTDirectoryNotFoundException extends IOException {
	private static final long serialVersionUID = 2543153762500385784L;
	
	/**
	 * Constructs an ENTDirectoryNotFoundException with null as its error detail message. 
	 */
	ENTDirectoryNotFoundException(){
		super();
	}
	
	/**
	 * Constructs an ENTDirectoryNotFoundException with the specified detail message. The error message string s can later be retrieved by the {@link java.lang.Throwable#getMessage() Throwable.getMessage()} method of class java.lang.Throwable.
	 * @param s the detail message
	 */
	ENTDirectoryNotFoundException(String s){
		super(s);
	}
}

/**
 * Signals that an attempt to open the file denoted by a specified pathname has failed.
 */
class ENTFileNotFoundException extends FileNotFoundException {
	private static final long serialVersionUID = 2543153762500385784L;
	
	/**
	 * Constructs an ENTFileNotFoundException with null as its error detail message. 
	 */
	ENTFileNotFoundException(){
		super();
	}
	
	/**
	 * Constructs an ENTFileNotFoundException with the specified detail message. The error message string s can later be retrieved by the {@link java.lang.Throwable#getMessage() Throwable.getMessage()} method of class java.lang.Throwable.
	 * @param s the detail message
	 */
	ENTFileNotFoundException(String s){
		super(s);
	}
}

/**
 * Thrown to indicate that a method has been passed a directory instead of a file, or reciprocally.
 */
class ENTInvalidFS_ElementTypeException extends IOException {
	private static final long serialVersionUID = 763310137094181767L;

	/**
	 * Constructs an ENTInvalidFS_ElementTypeException with null as its error detail message. 
	 */
	public ENTInvalidFS_ElementTypeException() {
		super();
	}

	/**
	 * Constructs an ENTInvalidFS_ElementTypeException with the specified detail message. The error message string s can later be retrieved by the {@link java.lang.Throwable#getMessage() Throwable.getMessage()} method of class java.lang.Throwable.
	 * @param s the detail message
	 */
	public ENTInvalidFS_ElementTypeException(String s) {
		super(s);
	}
}

/**
 *Thrown to indicate that a user's authentication control has failed (i.e. user isn't authenticated or session has expired).
 */
class ENTUnauthenticatedUserException extends IllegalStateException {
	
	private static final long serialVersionUID = -6029089402020384239L;
	public static final int UNAUTHENTICATED = 0;
	public static final int SESSION_EXPIRED = 1;
	private int type = UNAUTHENTICATED;

	/**
	 * Constructs an ENTUnauthenticatedUserException with null as its error detail message and unauthenticated as its detail code. 
	 */
	public ENTUnauthenticatedUserException() {
		super();
	}

	/**
	 * Constructs an ENTUnauthenticatedUserException with the specified detail message and unauthenticated as its detail code. The error message string s can later be retrieved by the {@link java.lang.Throwable#getMessage() Throwable.getMessage()} method of class java.lang.Throwable.
	 * @param s the detail message
	 */
	public ENTUnauthenticatedUserException(String s) {
		super(s);
	}
	
	/**
	 * Constructs an ENTUnauthenticatedUserException with the specified detail message and detail code. The error message string s can later be retrieved by the {@link java.lang.Throwable#getMessage() Throwable.getMessage()} method of class java.lang.Throwable.
	 * @param s the detail message
	 * @param type the detail code
	 */
	public ENTUnauthenticatedUserException(String s, int type) {
		super(s);
	}
	
	/**
	 * Constructs an ENTUnauthenticatedUserException with null as its error detail message and the specified detail code. 
	 * @param type the detail code
	 */
	public ENTUnauthenticatedUserException(int type) {
		super();
	}

	/**
	 * @return The detail code of the exception
	 */
	public int getType() {
		return type;
	}
}