package entDownloader;

import java.util.Calendar;
import java.util.GregorianCalendar;

public abstract class FS_Element {
	
	private GregorianCalendar dateModif;
	private String name;
	//Element type constants
	public static final short DIRECTORY = 0;
	public static final short FILE = 1;
	
	/**
	 * Construit un nouvel lment de systme de fichiers (dossier ou fichier)
	 * @param name Nom de l'lment
	 * @param dateModif Date de modification de l'lment
	 */
	FS_Element(String name, GregorianCalendar dateModif) {
		this.name = name;
		setDateModif(dateModif);
	}

	/**
	 * @return La date de modification de l'lment
	 */
	public GregorianCalendar getDateModif() {
		return dateModif;
	}

	/**
	 * @param dateModif Dfinit la date de modification de l'lment.
	 */
	public void setDateModif(GregorianCalendar dateModif) {
		this.dateModif = dateModif;
	}

	/**
	 * @return Le nom de l'lment
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name Le nouveau nom de l'lment
	 */
	public void setName(String name) {
		this.name = name;
	}
	
	@Override public String toString() {
		return "\n" + getName() + "\t" + dateModif.get(Calendar.DATE) + "-" + ((dateModif.get(Calendar.MONTH)==0)?12:dateModif.get(Calendar.MONTH)) + "-" + dateModif.get(Calendar.YEAR) + " " + dateModif.get(Calendar.HOUR_OF_DAY) + ":" + dateModif.get(Calendar.MINUTE);
	}
	
	/*@Override public boolean equals(Object obj){
		if(obj == null) return false;
		return true;
	}*/
	
	abstract short getType();
	
	public boolean isFile() {
		return getType() == FILE;
	}
	
	public boolean isDirectory() {
		return getType() == DIRECTORY;
	}

	/**
	 * @see java.lang.Object#hashCode()
	 */
	@Override public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((dateModif == null) ? 0 : dateModif.hashCode());
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		return result;
	}

	/**
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (obj instanceof String && obj.equals(name))
			return true;
		else if (getClass() != obj.getClass())
			return false;
		FS_Element other = (FS_Element) obj;
		if (name == null) {
			if (other.name != null)
				return false;
		} else if (!name.equals(other.name))
			return false;
		if (dateModif == null) {
			if (other.dateModif != null)
				return false;
		} else if (!dateModif.equals(other.dateModif))
			return false;
		return true;
	}
}
